/*
    netsetup.c
    Networking parameters set-up program for libsocket, Version 1.13
    Copyright 1997-2000 by Richard Dawe

    ----

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/*
 * TODO:
 *
 * . Make the slashes in printed filenames consistent - don't mix \ & /.
 *
 * . Offer the choice of backing up config files if they exist, in addition
 *   to overwriting them.
 *
 */

#include <conio.h>
#include <limits.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/stat.h>

#include <arpa/inet.h>
#include <netinet/in.h>
#include <resolv.h>

#define TRUE -1
#define FALSE 0

void banner (void);

/* --------
   - main -
   -------- */

int main (void)
{
    int  dns_present;           /* Boolean                */
    char host_name[256];        /* Host name              */
    char host_name_short[256];  /* Host name without dom. */
    char domain_name[256];      /* Domain name            */
    char *lsck_dir = NULL;      /* libsocket's config dir */
    char filename[PATH_MAX];    /* Temp. filename         */

    struct in_addr ip_addr, net_mask, net_addr, dns_addr[MAXNS + 1];
    char net_addr_str[16];      /* Net address in string  */

    char line[256];

    FILE *fOut; /* Output file handle */
    char *p, *q;
    int found;
    int i, j, k;

    bzero(&ip_addr,  sizeof(ip_addr));
    bzero(&net_mask, sizeof(net_mask));
    bzero(&net_addr, sizeof(net_addr));
    bzero(&dns_addr, sizeof(dns_addr));

    /* Banner */
    banner();

    /* --- Ask the questions --- */

    /* - libsocket configuration info - */
    printf("* libsocket Configuration Directory *\n\n");

    lsck_dir = getenv("LSCK");

    if (lsck_dir == NULL) {
        lsck_dir = getenv("LIBSOCKET");
        if (lsck_dir != NULL)
            printf("WARNING: Please use LSCK environment variable "
                   "instead of LIBSOCKET.\n");
    }        

    printf("The libsocket environment variable, LSCK, specifies the directory "
           "where\nlibsocket's configuration files are stored.\n\n");

    if (lsck_dir == NULL) {
        printf("LSCK has not been set. Do you wish to set it up [Yn]? ");
        fflush(stdout);
        scanf("%1s", line);

        if ((*line == 'Y') || (*line == 'y')) {
            printf("Please enter the libsocket configuration directory "
                   "(e.g. C:\\LSCK):\n");
            fflush(stdout);
            scanf("%255s", line);

            /* Set up the new var */
            p = line + strlen(line) - 1;    /* Trailing slash? */
            if ((*p == '\\') || (*p == '/')) *p = '\0';

            lsck_dir = strdup(line);
            setenv("LSCK", lsck_dir, 1);

            /* Add to AUTOEXEC.BAT? */
            printf("Add the line 'SET LSCK=%s' to C:\\AUTOEXEC.BAT [Yn]? ",
                    lsck_dir);
            fflush(stdout);
            scanf("%1s", line);

            if ((*line == 'Y') || (*line == 'y')) {
                fOut = fopen("c:\\autoexec.bat", "at");
                if (fOut != NULL) {
                    fprintf(fOut, "SET LSCK=%s\n", lsck_dir);
                    fclose(fOut);
                }
            }

            /* Create the directory */
            mkdir(lsck_dir, S_IWUSR);
        } else {
            printf("netsetup needs the configuration directory to be set!\n");
            exit(1);
        }
    } else {      
        printf("The libsocket configuration directory is '%s'.\n", lsck_dir);
    }

    /* - Host & domain name - */
    printf("\n"
           "* Host and domain name *\n\n"
           "These are required if you wish to use text names rather than IP "
           "addresses.\n\n");

    /* Host name */
    printf("Please enter your computer's host name: ");
    fflush(stdout);
    scanf("%255s", host_name);

    /* Build a suggested domain name from the host name */
    p = strchr(host_name, '.');
    if (p != NULL) {
        strcpy(domain_name, p + 1);
        printf("Using '%s' as the domain name.\n", domain_name);
    } else {        
        /* Domain name */
        printf("Do you have a domain name [Yn]? ");
        fflush(stdout);
        scanf("%1s", line);

        if ((*line == 'Y') || (*line == 'y')) {
            printf("Please enter your computer's domain name: ");
            fflush(stdout);
            scanf("%255s", domain_name);

            /* Now add this to the host name */
            strcat(host_name, ".");
            strcat(host_name, domain_name);
            printf("Now using '%s' as host name.\n", host_name);
        }
    }

    /* - IP address details - */
    printf("\n"
           "* IP Address Information *\n\n"
           "This is required for IP networking and text name to IP address "
           "mappings\n(name resolving).\n\n");

    /* IP address */
    printf("Please enter your computer's IP address (e.g. 192.168.1.123):\n");
    fflush(stdout);
    scanf("%255s", line);
    inet_aton(line, &ip_addr);

    /* Network mask */
    printf("Please enter your computer's IP network mask "
	   "(e.g. 255.255.255.0):\n");
    fflush(stdout);
    scanf("%255s", line);
    inet_aton(line, &net_mask);

    /* Make network address from network mask */
    net_addr.s_addr = ip_addr.s_addr & net_mask.s_addr;
    printf("Using %s as the network address.\n\n", inet_ntoa(net_addr));

    /* DNS details */
    bzero(dns_addr, sizeof(dns_addr));

    printf("Do you have a DNS server [Yn]? ");
    fflush(stdout);
    scanf("%1s", line);
    dns_present = ((*line == 'Y') || (*line == 'y')) ? TRUE : FALSE;

    if (dns_present == TRUE) {
      for (i = 0; i < MAXNS; i++) {
	printf ("Please enter the IP address DNS server %i "
		"(enter for none): ", i + 1);
	fflush(stdout);
	scanf("%255s", line);
	if (line[0] == '\0') break;
	inet_aton(line, &dns_addr[i]);
      }
    }

    /* --- Create the files --- */
    
    /* Installation message */
    printf("\n"
           "* Creating Files *\n\n");

    /* - lsck.cfg - */
    sprintf(filename, "%s\\lsck.cfg", lsck_dir);
    if (access(filename, F_OK) == 0) {
        printf("%s already exists - overwrite [yN]? ", filename);
        fflush(stdout);
        scanf("%1s", line);
        if ((*line == 'N') || (*line == 'n')) exit(1);
    }
    printf("Creating %s...", filename);

    fOut = fopen(filename, "wt");
    if (fOut == NULL) {
        printf("failed\n");
        perror("netsetup");
    } else {
        /* Header */
        fprintf(fOut,
                "# lsck.cfg\n"
                "# libsocket configuration file\n"
                "# Created by netsetup\n"
                "#\n\n");

        /* Host & domain name */
        fprintf(fOut,
                "[main]\n"
                "hostname=%s\n\n",
                host_name);

        /* As libsocket only runs under Windows, use Windows config files */
        fprintf(fOut,
                "# Use some Windows configuration files\n"
                "hosts=%s\\hosts\n"
                "networks=%s\\networks\n"
                "services=%s\\services\n"
                "protocols=%s\\protocol\n\n",
                getenv("windir"), getenv("windir"),
                getenv("windir"), getenv("windir"));

        /* Resolver config files */
        fprintf(fOut,
                "# Resolver configuration files\n"
                "host.conf=%s\\host.cfg\n"
                "resolv.conf=%s\\resolv.cfg\n\n",
                lsck_dir, lsck_dir);

        /* Global remote auto-login files */
        fprintf(fOut,
                "# Global remote auto-login files\n"
                "hosts.equiv=%s\\hosts.equ\n"
                ".rhosts=%s\\rhosts\n"
                ".netrc=%s\\netrc\n\n",
                lsck_dir, lsck_dir, lsck_dir);

        /* Done */
        printf("done\n");
        fclose(fOut);
    }

    /* - hosts - */    
    strcpy(host_name_short, host_name);
    p = strchr(host_name_short, '.');
    if (p != NULL) *p = '\0';

    p = inet_ntoa(ip_addr);

    sprintf(filename, "%s\\hosts", getenv("windir"));
    printf("Modifying %s...", filename);

    fOut = fopen(filename, "a+t");
    if (fOut == NULL) {
        printf("failed\n");
        perror("netsetup");
    } else {
        rewind(fOut);
        found = 0;

        /* Look for the IP address & host name */
        while(fgets(line, sizeof(line), fOut) != NULL) {
            q = line;
            while ((*q == ' ') || (*q == '\t')) q++;
            if (*q == '#') continue;

            if (   (strstr(q, p)               != NULL)
                && (strstr(q, host_name_short) != NULL)) {
                found = 1;
                break;
            }
        }

        fseek(fOut, 0, SEEK_END);
        if (!found) fprintf(fOut, "%s    %s\n", p, host_name_short);

        /* Done */
        if (!found)
	   printf("done\n");
	else
	   printf("done (no changes needed)\n");
        fclose(fOut);
    }

    /* - networks - */

    /* Build the network mask in string form */
    bzero(net_addr_str, sizeof(net_addr_str));
    for (i = 3; i > 0; i--) {
        j = (ntohl(net_addr.s_addr) >> (i * 8)) & 0xFF;
        k = (ntohl(net_mask.s_addr) >> (i * 8)) & 0xFF;
        if (k != 0) {
            itoa(j, line, 10);
            if (i < 3) strcat(net_addr_str, ".");
            strcat(net_addr_str, line);
        }
    }

    sprintf(filename, "%s\\networks", getenv("windir"));
    printf("Modifying %s...", filename);

    fOut = fopen(filename, "a+t");
    if (fOut == NULL) {
        printf("failed\n");
        perror("netsetup");
    } else {
        rewind(fOut);
        found = 0;

        /* No domain name => this isn't needed, so skip it */
        if (*domain_name == '\0') found = 1;

        /* Look for the IP address & host name */
        while(fgets(line, sizeof(line), fOut) != NULL) {
            q = line;
            while ((*q == ' ') || (*q == '\t')) q++;
            if (*q == '#') continue;

            if (   (strstr(q, net_addr_str) != NULL)
                && (strstr(q, domain_name)  != NULL)) {
                found = 1;
                break;
            }
        }

        fseek(fOut, 0, SEEK_END);
        if (!found) fprintf(fOut, "%s    %s\n", net_addr_str, domain_name);

        /* Done */
        if (!found) printf("done\n"); else printf("done (none needed)\n");
        fclose(fOut);
    }

    /* - host.cfg - */
    sprintf(filename, "%s\\host.cfg", lsck_dir);
    if (access(filename, F_OK) == 0) {
        printf("%s already exists - overwrite [yN]? ", filename);
        fflush(stdout);
        scanf("%1s", line);
        if ((*line == 'N') || (*line == 'n')) exit(1);
    }
    printf("Creating %s...", filename);

    fOut = fopen(filename, "wt");
    if (fOut == NULL) {
        printf("failed\n");
        perror("netsetup");
    } else {
        fprintf(fOut,
                "# host.conf\n"
                "# Resolver configuration file\n"
                "# Created by netsetup\n\n");

        if (dns_present == TRUE)
            fprintf(fOut, "order hosts, bind\n");
        else
            fprintf(fOut, "order hosts\n");

        /* Done */
        printf("done\n");
        fclose(fOut);
    }

    /* - resolv.cfg - */
    sprintf(filename, "%s\\resolv.cfg", lsck_dir);
    if (access(filename, F_OK) == 0) {
        printf("%s already exists - overwrite [yN]? ", filename);
        fflush(stdout);
        scanf("%1s", line);
        if ((*line == 'N') || (*line == 'n')) exit(1);
    }
    printf("Creating %s...", filename);

    fOut = fopen(filename, "wt");
    if (fOut == NULL) {
        printf("failed\n");
        perror("netsetup");
    } else {
        fprintf(fOut,
                "# resolv.conf\n"
                "# Resolver configuration file\n"
                "# Created by netsetup\n\n");

        if (dns_present == TRUE) {
	  for (i = 0; i < MAXNS; i++) {
	    if (dns_addr[i].s_addr == 0) break;
            fprintf(fOut, "nameserver %s\n", inet_ntoa(dns_addr[i]));
	  }
	}

        /* Done */
        printf("done\n");
        fclose(fOut);
    }

	/* Success */
	return (0);
}

/* ----------
   - banner -
   ---------- */

void banner (void)
{
    /* Copyright */
    printf ("Netsetup Version 1.13, libsocket networking set-up\n"
            "Copyright 1997-2000 by Richard Dawe\n");

    printf ("libsocket Copyright 1997, 1998 by Indrek Mandre\n"
            "libsocket Copyright 1997-2000 by Richard Dawe\n\n");

    /* Info message */
    printf("Netsetup will ask you a series of questions about your TCP/IP "
           "configuration. It\nmay help you to look at the TCP/IP settings "
           "in the Network Control Panel or run\n'winipcfg'. See the "
           "libsocket documentation for further help.\n\n");

    fflush(stdout);
}
